source("start.R")
source("include-classification-model-specification.R")
source("include-classification.R")
source("include-marginal-logic.R")

#for (i in 1:length(dataset.names)) {
for (i in 2) {
  dataset.name <- dataset.names[i]  

  cat("Starting classification for dataset \"", dataset.name, "\".\n", sep="")

  # Global assignment: hack, but easier...
  loci.count <- get.loci.count(dataset.name)
  loci <- 1:loci.count
  loci.names <- get.loci.names(dataset.name)

  dataset.compact <- get.compact.dataset(dataset.name)
  dataset.extended <- get.extended.dataset(dataset.name)
  observation.count <- nrow(dataset.extended)

  cat("Number of observations = ", observation.count, "\n", sep="")

  #for (j in 1:length(classification.model.use)) {
  for (j in 1) {
    m <- classification.model.use[j]
    classification.model.name <- classification.model.names[m]
    classification.model.function <- classification.model.functions[[m]]
    classification.model.score <- classification.model.scores[[m]]
    classification.model.prediction <- classification.model.predictions[[m]]

    cat("Using classification method \"", classification.model.name, "\".\n", sep="")

    model.fits <- NULL
    
    resultTry <- try(
      model.fits <- get.best.model.fits(dataset.name,
        classification.model.name,
        classification.model.function, 
        classification.model.score)
    )

    if (is.null(model.fits) || inherits(resultTry, "try-error")) {
      unobserved.prob.matrix[i, j+2] <- NA
      cat("Method \"", classification.model.name, "\" failed.\n\n", sep="")
      next
    } 

    locus.not.used <- setdiff(loci, model.fits$loci.used)

    cat("===============================\n")
    cat("Observed haplotype count table\n")
    print(get.haplotype.count.table(dataset.name))
    cat("===============================\n")
  
    for (k in 1:10) {
      dataset.simulated <- simulate.haplotypes(dataset.name, model.fits,
        simulations = observation.count,
        classification.model.name,
        classification.model.prediction)

      print(table(table(apply(dataset.simulated, 1, paste, collapse=","))))
    }

    cat("Method \"", classification.model.name, "\" done.\n\n", sep="")
  }

  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")
}

