source("start.R")

# do.replacements is in here:
source("include-marginal-logic.R")

library(xtable)

for (i in 1:length(dataset.names)) {
  dataset.name <- dataset.names[i]
  loci.names <- get.loci.names(dataset.name)

  cat("Starting finding correlations for dataset \"", dataset.name, "\".\n", sep="")

  methodNames <- c("pearson", "kendall", "spearman")
  #methodNames <- c("pearson", "kendall")
  #methodExacts <- c(NULL, NULL, FALSE)
  methodExacts <- c(TRUE, TRUE, FALSE)
  methodLabels <- c("Pearson's correlation coefficient", "Kendall's $\\tau$", "Spearman's $\\rho$")

  for (j in 1:length(methodNames)) {  
    methodName <- methodNames[j]
    methodLabel <- methodLabels[j]
    methodExact <- methodExacts[j]

    corMatrix <- get.correlation.matrix(dataset.name, exact.p.value = methodExact, method=methodName)
    colnames(corMatrix) <- paste("¤LOCUS:", colnames(corMatrix), "¤", sep="")
    rownames(corMatrix) <- colnames(corMatrix)
    
    #corMatrix[upper.tri(corMatrix)] <- NA    
    #rownames(corMatrix) <- paste("\textbf{", rownames(corMatrix), "}")
    #colnames(corMatrix) <- paste("\textbf{", colnames(corMatrix), "}")    

    tableOutput <- xtable(corMatrix,
      label=paste("tab:correlation-matrix-", methodName, "-", dataset.name, sep=""), 
      caption=paste("Correlation matrix for ¤DATASET:", dataset.name, "¤ using ", methodLabel, ". The lower triangular matrix shows the actual correlations, and the upper triangular matrix shows the two-sided $p$-values from testing if the correlations are equal against being different.", sep=""),
      digits=3) 

    filename <- paste("out/correlation-matrix-", methodName, "-", dataset.name, ".tex", sep="")
    print(tableOutput, file=filename)
    do.replacements(filename)
  }
  
  cat("Dataset \"", dataset.name, "\" done.\n", sep="")
}

