print.simultanous <- function(h) {
  cat("P(");
  
  for (i in 1:loci.count) {
    k <- loci[i]
    cat(loci.names[k], "=", h[k])
    if (i < loci.count) cat(", ") 
  }
  
  cat(")")
}

print.conditionals <- function(h, ancestors) {
  ancestors.count <- length(ancestors)
  
  if (ancestors.count > 0) {
    for (i in 1:ancestors.count) {
      j <- ancestors[i]
      
      cat(loci.names[j], "=", h[j])    
      if (i < ancestors.count) cat(", ")
    }
  }
}

# Used to print probabilities, both conditional and unconditional, e.g.:
# P(DYS19 = 13)
# P(DYS19 = 13 | DYS393 = 13)
# P(DYS19 = 13 | DYS393 = 13, DYS437 = 15)
# etc.
print.probability <- function(h, k, ancestors) {
  ancestors.count <- length(ancestors)
  
  cat(sprintf("P(%s = %s", loci.names[k], h[k]))
  
  if (ancestors.count > 0) {
    cat(" | ", sep="")
    print.conditionals(h, ancestors)
  }
  
  cat(")")
}

# Used to print probabilities, both conditional and unconditional, e.g.:
# P(DYS19 = 13) = 0.444
# P(DYS19 = 13 | DYS393 = 13) = 0.333
# P(DYS19 = 13 | DYS393 = 13, DYS437 = 15) = 0.888
# etc.
print.probability.with.result <- function(h, k, k.prob, ancestors) {
  ancestors.count <- length(ancestors)
  
  cat(sprintf("P(%s = %s", loci.names[k], h[k]))
  
  if (ancestors.count > 0) {
    cat(" | ", sep="")
    print.conditionals(h, ancestors)
  }
  
  cat(sprintf(") = %.3f", k.prob))
}

# Used to print probabilities, both conditional and unconditional, e.g.:
# P(   DYS19 = 13) = 0.041
# P(   DYS19 = 13 | DYS393 = 13) = 0.054
# P(   DYS19 = 13 | DYS393 = 13, DYS437 = 15) = 0.043
# etc.
print.probability.nice.with.result <- function(h, k, k.prob, ancestors) {
  ancestors.count <- length(ancestors)
  
  cat(sprintf("P(%8s = %2s", loci.names[k], h[k]))
  
  if (ancestors.count > 0) {
    cat(" | ", sep="")
    print.conditionals(h, ancestors)
  }
  
  cat(sprintf(") = %.3f\n", k.prob))
}

print.approx.probability <- function(h, ancestors, ancestors.probs) {
  ancestors.count <- length(ancestors)
  
  if (ancestors.count <= 0) stop("ancestors.count <= 0 not expected in print.approx.probability, nor in algorithm in general at this time of execution")

  loci.left <- loci
  
  if (ancestors.count > 0) {
    loci.left <- loci.left[-ancestors]
  }
  
  loci.left.count <- length(loci.left)
  
  cat("Define the event\nC := {")
  print.conditionals(h, ancestors)
  cat("}\n")
  
  cat("Then\n")
  
  print.simultanous(h)
  
  cat("\n = P(")
  
  for (i in 1:loci.left.count) {
    k <- loci.left[i]
    cat(loci.names[k], "=", h[k])
    if (i < loci.left.count) cat(", ") 
  }
    
  cat(" | C)\n   * ")

  # (1): Same as (2) ->
  for (i in 1:ancestors.count) {
    k <- ancestors[i]
    
    if (i == 1) {
      print.probability(h, k, c())
    } else {
      cat("\n   * ")
      print.probability(h, k, ancestors[1:(i-1)])
    }
  }
  # <- (1): Same as (2)
  
  cat("\n ≈ ")
  
  for (i in 1:loci.left.count) {
    k <- loci.left[i]
       
    cat("P(", loci.names[k], " = ", h[k], " | C)", sep="")    
    
    if (i < loci.left.count) cat(" * ") 
  }
  
  cat("\n   * ")
  
  # (2): Same as (1) ->
  for (i in 1:ancestors.count) {
    k <- ancestors[i]
    
    if (i == 1) {
      print.probability(h, k, c())
    } else {
      cat("\n   * ")
      print.probability(h, k, ancestors[1:(i-1)])
    }
  }
  # <- (2): Same as (1)
   
  cat("\n = ")
  
  print.approx.probability.independent.part(h, ancestors)
 
  ancestors.probs.length <- length(ancestors.probs)
  for (i in 1:ancestors.probs.length) {
    cat("\n   *", ancestors.probs[i])
  }
  
  cat("\n")
}

print.approx.probability.independent.part <- function(h, ancestors) {
  ancestors.count <- length(ancestors)

  # FIXME: Maybe a check and stop if ancestors.count <= 0

  observations.left <- get.observations.left(h, ancestors)
  loci.left <- get.loci.left(ancestors)    
  loci.left.count <- length(loci.left)
  
  for (j in 1:loci.left.count) {
    k <- loci.left[j]
    tab <- table(dataset[observations.left, k])
     
    # Which index in the table are we interested in?
    i <- which(as.data.frame(tab)[,1] == h[k])
       
    # If the locus wasn't found, no worries: probs[k] is already 0
    if (length(i) == 1) {
      tab.prob <- as.data.frame(prop.table(tab))
      cat(tab.prob[i, 2])
      
      if (j < loci.left.count) cat(" * ")
    } else {
      #print(tab)
      # FIXME: Should this be un-out-commented?
      #stop("Locus ", loci.names[k], " didn't had allele ", h[k], " in the haplotypes left", sep="")
    }   
  }  
}

calc.approx.probability.conditional.prod <- function(h, ancestors, ancestors.probs) {
  ancestors.count <- length(ancestors)

  # FIXME: Maybe a check and stop if ancestors.count <= 0

  observations.left <- get.observations.left(h, ancestors)
  loci.left <- get.loci.left(ancestors)
  
  prob <- prod(ancestors.probs)
  
  for (k in loci.left) {
    tab <- table(dataset[observations.left, k])
     
    # Which index in the table are we interested in?
    i <- which(as.data.frame(tab)[,1] == h[k])
       
    # If the locus wasn't found, no worries: probs[k] is already 0
    if (length(i) == 1) {
      tab.prob <- as.data.frame(prop.table(tab))      
      prob <- prob * tab.prob[i, 2]
    } else {
      #print(tab)
      # FIXME: Should this be un-out-commented?
      #stop("Locus ", loci.names[k], " didn't had allele ", h[k], " in the haplotypes left", sep="")
    }   
  }
  
  return(prob)
}

get.observations.left <- function(h, ancestors) {
  ancestors.count <- length(ancestors)
  
  observations.left <- 1:observation.count
  
  if (ancestors.count > 0) {
    for (k in ancestors) {
      observations.left <- intersect(observations.left, which(dataset[,k] == h[k]))
    }
  }
  
  return(observations.left)
}

get.loci.left <- function(ancestors) {
  ancestors.count <- length(ancestors)
  
  loci.left <- loci
  
  if (ancestors.count > 0) {
    loci.left <- loci.left[-ancestors]
  }
  
  return(loci.left)
}

# k: which component in h
# observations.left is the implementation of the conditional probabilities
get.single.prob <- function(h.index, h.value, observations.left, loci.left) {
  tab <- table(dataset[observations.left, h.index])
     
  # Which index in the table are we interested in?
  i <- which(as.data.frame(tab)[,1] == h.value)
    
  # If the locus wasn't found, no worries: return(0) at the end
  if (length(i) == 1) {
    tab.prob <- as.data.frame(prop.table(tab))
    return(tab.prob[i, 2])
  }
   
  return(0)
}

get.probs <- function(h, ancestors) {
  observations.left <- get.observations.left(h, ancestors)
  loci.left <- get.loci.left(ancestors)
 
  probs <- rep(0, loci.count)

  # One could find the max directly in the loop instead, but
  # this approach makes it easier to debug and change the code behaviour
  for (k in loci.left) {
    probs[k] <- get.single.prob(k, h[k], observations.left, loci.left)
    
    if (!SUPRESS_LOG) {
      print.probability.nice.with.result(h, k, probs[k], ancestors)
    }
  }
  
  #if (all(probs == 0)) {
  #  stop("all(probs == 0)")
  #}
  
  return(probs)
}

get.possible.mutations <- function(h.index, h.value) {
  locus.levels <- loci.levels[[h.index]]
  locus.levels.length <- length(locus.levels)
  
  mutation.alternatives <- c()
  
  # If the chosen allele is the only no, then there's no
  # idea in trying to find alternatives
  if (locus.levels.length > 1) {
    index <- which(h.value == locus.levels)
    
    if (index > 1) {
      mutation.alternatives <- c(mutation.alternatives, locus.levels[index - 1])
    }
    
    if (index < locus.levels.length) {
      mutation.alternatives <- c(mutation.alternatives, locus.levels[index + 1])
    }
  }
  
  return(mutation.alternatives)
}

# Given a haplotype h, this greedy algorithm finds an ancestor with the 
# highest probability until a maximum number of ancestors has been found.
# FIXME: Maybe stop at a probability threshold instead?
# ancestor.limit defined and observations.left.threshold = NULL => use ancestor.limit
# ancestor.limit = NULL and observations.left.threshold is defined => use observations.left.threshold
# ancestor.limit defined and observations.left.threshold is defined => use both!
# both ancestor.limit = NULL and observations.left.threshold = NULL => error
calc.approx.probability <- function(h, ancestors = c(), ancestors.probs = c(), ancestor.limit = 3, observations.left.threshold = NULL) {
  # Just an trivial error check
  if (length(h) != length(loci)) {
    stop("The vector h doesn't have the right length!")
  }
  
  if (is.null(ancestor.limit) && is.null(observations.left.threshold)) {
    stop("Not both ancestor.limit and observations.left.threshold can be NULL")
  }
  
  if (!is.null(observations.left.threshold) && (observations.left.threshold < 0 || observations.left.threshold > 1)) {
    stop("observations.left.threshold must lie in [0, 1]")
  }
  
  ancestors.count <- length(ancestors)  
  
  observations.left <- get.observations.left(h, ancestors)  
  observations.left.length <- length(observations.left)  
  observations.left.proportion <- observations.left.length / observation.count  

  # First time called by user and not by recursion
  if (!SUPRESS_LOG && ancestors.count == 0) {
    cat("\n\n################################\n")
    cat("Haplotype:")
    print(h)
    cat("################################\n")
  }
    
  if (!SUPRESS_LOG) {
    cat("================================\n")
    cat(" calc.approx.probability called \n")
    cat("================================\n")
    
    cat(ancestors.count, "ancestors so far:\n")
    print(ancestors)
    
    cat("Number of observations left =", observations.left.length, "\n")
  }    
  
  # 0: not done
  # 1: ancestor.limit
  # 2: observations.left.threshold
  # 3: all but one is ancestors
  done.reason <- 0 
  
  if (!is.null(ancestor.limit) && ancestors.count >= ancestor.limit) {
    done.reason <- 1
  }
    
  if (!is.null(observations.left.threshold) && observations.left.proportion < observations.left.threshold) {
    done.reason <- 2
  }
    
  if (ancestors.count >= loci.count-1) {
    done.reason <- 3
  }
  
  # Check if we're done
  if (done.reason > 0) {
    prob <- calc.approx.probability.conditional.prod(h, ancestors, ancestors.probs)
    
    if (!SUPRESS_LOG) {
      cat("Observations left:\n")
      print(dataset[observations.left, loci])
      
      if (done.reason == 1) {
        cat("The limit of the number of found ancestors, ", ancestor.limit, ", has been reached, hence we stop.\n", sep="")
      } else if (done.reason == 2) {
        cat("The proportion of observations left, ", round(observations.left.proportion, 3), ", has exceeded the threshold, ", observations.left.threshold, ", hence we stop.\n", sep="")
      } else if (done.reason == 3) {
        cat("All but one locus is now ancestors, hence we stop.\n", sep="")
      }
    
      cat("\n-------------------------\n\n")

      print.approx.probability(h, ancestors, ancestors.probs)
  
      cat(" =", prob, "\n")
    }
    
    return(prob)
  }
  
  loci.left <- get.loci.left(ancestors)
  
  if (!SUPRESS_LOG) {
    cat("Number of loci left =", length(loci.left), "\n")
  }

  probs <- get.probs(h, ancestors)
  
  max.prob.index <- which.max(probs)
  max.prob <- probs[max.prob.index]
  nm <- loci.names[max.prob.index]
  
  if (!SUPRESS_LOG) {
    cat("Locus no. ", max.prob.index, ", i.e. ", nm, ", gives ", sep="")
    print.probability.with.result(h, max.prob.index, max.prob, ancestors)
    cat(" which is the highest probability found.\n\n", sep="")
  }
  
  calc.approx.probability(h, c(ancestors, max.prob.index), c(ancestors.probs, max.prob), ancestor.limit, observations.left.threshold)
}

calc.approx.probability.with.mutation <- function(h, ancestors = c(), ancestors.probs = c(), ancestor.limit = 3, observations.left.threshold = NULL) {
  prob <- calc.approx.probability(h, ancestors, ancestors.probs, ancestor.limit, observations.left.threshold)
  
  # Now find single step mutations of h
  h.length <- length(h)
  for (j in 1:loci.count) {
    k <- loci[j]
    mutations <- get.possible.mutations(k, h[k])
    mutations.length <- length(mutations)
      
    probs.tmp <- numeric(mutations.length)
      
    for (i in 1:mutations.length) {          
      a <- mutations[i]
      
      h.tmp <- h
      h.tmp[k] <- a

      if (!SUPRESS_LOG) {
        cat("\n\nTrying mutation on ", j, ". locus\n", sep="")
      }
      
      probs.tmp[i] <- calc.approx.probability(h.tmp, ancestors, ancestors.probs, ancestor.limit, observations.left.threshold)
     
      prob.mut <- max(probs.tmp)
      
      if (prob.mut > prob) {
        if (!SUPRESS_LOG) {
          cat("Mutation from ", loci.names[k], " = ", h[k], " to ", loci.names[k], " = ", a," payed off with P(...) = ", prob.mut, " instead of P(...) = ", prob, "\n", sep="")
          print(h.tmp)
        }
        
        prob <- prob.mut
      } # if (prob.mut > prob)
    } # for (i in 1:mutations.length)
  } # for (j in 1:loci.count)
  
  return(prob) 
}

#SUPRESS_LOG <- FALSE
#h <- c(13, 13, 29, 22, 10, 6, 13, 14, 11, 12)
#sink("ancestor-mut.log")
#calc.approx.probability.with.mutation(h)
#sink()

