get.formula.custom <- function(response, explanatories) {
  return(paste(loci.names[response], "~", paste(loci.names[explanatories], collapse=" + ")))
}

######################

get.best.model.fits <- function(dataset.name,
  classification.model.name,
  classification.model.function, 
  classification.model.score) {

  dataset.extended <- get.extended.dataset(dataset.name)
  
  used.loci <- c()
  used.formulas <- c()
  used.scores <- c()
  used.fits <- vector("list", loci.count-1)

  for (k in 1:(loci.count-1)) {
    cat("=======================================\n")
    cat("k =", k, "\n")
    cat("=======================================\n")
    
    used.loci.length <- length(used.loci)
    loci.left <- loci
    
    if (used.loci.length > 0) {
      print(loci.left)
      print(used.loci)
      loci.left <- loci.left[-used.loci]
    }
    
    loci.left.length <- length(loci.left)

    model.fits <- vector("list", loci.left.length)
    formulas <- c()
    
    for (j in 1:loci.left.length) {      
      response <- loci.left[j]
      explanatories <- loci.left[-j]
      
      dataset.extended.mod <- dataset.extended[, loci.left]
      dataset.extended.mod[, j] <- factor(dataset.extended.mod[, j])
      
      #get.formula.custom(response, explanatories)
      formula <- get.formula.custom(response, explanatories)
      formulas <- c(formulas, formula)
      fit <- classification.model.function(as.formula(formula), data=dataset.extended.mod)
      model.fits[[j]] <- fit
      cat("Formula: ", formula, "\n")
      print(fit)
      cat("\n")
    } # for (j in 1:loci.left.length)    
      
    fit.scores <- classification.model.score(model.fits)
    print(fit.scores)
    min.score.index <- which.min(fit.scores)
    min.score <- fit.scores[min.score.index]
    min.score.loci <- loci.left[min.score.index]
    cat(formulas[min.score.index], " (j = ", min.score.index, ") gave the minimal score = ", min.score , "\n", sep="")
    cat("=======================\n")
    used.loci <- c(used.loci, min.score.loci)
    used.formulas <- c(used.formulas, formulas[min.score.index])
    used.scores <- c(used.scores, min.score)
    used.fits[[k]] <- model.fits[[min.score.index]]    
  } # for (k in 1:loci.count) 

  # Print summary data:
  for (k in 1:(loci.count-1)) {
    cat(used.formulas[k], "\n")
    cat("\tScore =", used.scores[k], "\n")
    cat("\n")
  }
    
  return(list(fits=used.fits, formulas=used.formulas, loci.used=used.loci, scores=used.scores))
}

# A modified copy of predict.probabilities so far only used in 
# classification-evidence-2-contributors.R
predict.haplotypes.probabilities <- function(dataset, dataset.single.marginals, model.fits,
  classification.model.name,
  classification.model.function, 
  classification.model.prediction) {

  dataset <- as.data.frame(dataset)
  
  haplotypes.count <- nrow(dataset)

  dataset.predicted <- cbind(dataset, prob=1)
  loci.used.length <- length(model.fits$loci.used)
  probs.calculations <- vector("list", haplotypes.count)

  for (i in 1:loci.used.length) {
    locus <- model.fits$loci.used[i]
    fit <- model.fits$fits[[i]]
    prediction <- classification.model.prediction(fit, dataset)
    prediction.alleles <- colnames(prediction)

    for (m in 1:haplotypes.count) {
      j <- which(prediction.alleles == dataset.predicted[m, locus])
      if (length(j) != 1) stop("length(j) != 1")
      prob <- prediction[m, j]
      dataset.predicted$prob[m] <- dataset.predicted$prob[m] * prob
      probs.calculations[[m]] <- c(probs.calculations[[m]], prob)
    }
  }

  locus.left <- setdiff(loci, model.fits$loci.used)

  if (length(locus.left) != 1) stop("length(locus.left) != 1")

  tab <- prop.table(table(dataset.single.marginals[, locus.left]))

  for (m in 1:haplotypes.count) {
    allele <- dataset.predicted[m, locus.left]
    prob <- as.numeric(tab[which(names(tab) == allele)])
    dataset.predicted$prob[m] <- dataset.predicted$prob[m] * prob
    probs.calculations[[m]] <- c(probs.calculations[[m]], prob)
  }

  return(dataset.predicted$prob)
}

predict.probabilities <- function(dataset.name, model.fits,
  classification.model.name,
  classification.model.function, 
  classification.model.prediction) {
  
  dataset.compact <- get.compact.dataset(dataset.name)  
  haplotypes.count <- get.haplotypes.count(dataset.name)
  dataset.extended <- get.extended.dataset(dataset.name)

  dataset.compact.predicted <- cbind(dataset.compact, prob=1)
  loci.used.length <- length(model.fits$loci.used)
  probs.calculations <- vector("list", haplotypes.count)

  for (i in 1:loci.used.length) {
    locus <- model.fits$loci.used[i]
    fit <- model.fits$fits[[i]]
    #prediction <- predict(fit, dataset.compact)
    prediction <- classification.model.prediction(fit, dataset.compact)    
    prediction.alleles <- colnames(prediction)

    for (m in 1:haplotypes.count) {
      j <- which(prediction.alleles == dataset.compact.predicted[m, locus])
      if (length(j) != 1) stop("length(j) != 1")
      prob <- prediction[m, j]
      dataset.compact.predicted$prob[m] <- dataset.compact.predicted$prob[m] * prob
      probs.calculations[[m]] <- c(probs.calculations[[m]], prob)
    }
  }

  locus.left <- setdiff(loci, model.fits$loci.used)

  if (length(locus.left) != 1) stop("length(locus.left) != 1")

  tab <- prop.table(table(dataset.extended[, locus.left]))

  for (m in 1:haplotypes.count) {
    allele <- dataset.compact.predicted[m, locus.left]  
    prob <- as.numeric(tab[which(names(tab) == allele)])
    dataset.compact.predicted$prob[m] <- dataset.compact.predicted$prob[m] * prob
    probs.calculations[[m]] <- c(probs.calculations[[m]], prob)
  }

  uncovered.probability.mass <- 1-sum(dataset.compact.predicted$prob)

  cat("Uncovered probability mass =", round(uncovered.probability.mass, 4), "\n")

  return(list(probs = dataset.compact.predicted$prob, 
    uncovered.prob.mass = uncovered.probability.mass))
}

simulate.haplotypes <- function(dataset.name, model.fits,
  simulations = 10,
  classification.model.name,
  classification.model.prediction) {
  
  dataset.extended <- get.extended.dataset(dataset.name)
  
  locus.left <- setdiff(loci, model.fits$loci.used)

  # This could maybe be made generic somewhere:
  alleles <- vector("list", loci.count)
  for (k in loci) {
    alleles[[k]] <- as.numeric(names(table(dataset.extended[, k])))
  }

  if (length(locus.left) != 1) stop("length(locus.left) != 1")

  probs <- as.vector(prop.table(table(dataset.extended[, locus.left])))

  first.locus <- sample(alleles[[locus.left]], simulations, replace = TRUE, prob=probs)

  dataset.simulated <- matrix(NA, nrow=simulations, ncol=loci.count)
  colnames(dataset.simulated) <- loci.names
  dataset.simulated[, locus.left] <- first.locus
  #dataset.simulated <- as.data.frame(dataset.simulated)
  dataset.simulated2 <<- dataset.simulated

  loci.used.length <- length(model.fits$loci.used)

  dependent.loci <- locus.left

  for (i in loci.used.length:1) {
    locus <- model.fits$loci.used[i]
    fit <- model.fits$fits[[i]]

    dataset.simulated.mod <- as.matrix(dataset.simulated[, dependent.loci])
    colnames(dataset.simulated.mod) <- loci.names[dependent.loci]
    dataset.simulated.mod <- as.data.frame(dataset.simulated.mod)
    
    prediction <- classification.model.prediction(fit, dataset.simulated.mod)

    dataset.simulated[, locus] <- apply(prediction, 1, function(p) sample(alleles[[locus]], 1, prob=p) )

    dependent.loci <- c(dependent.loci, locus)      
  }

  return(dataset.simulated)
}

