# N: vector of observed counts
# p: vector of predicted frequencies
deviance.relfreq.predicted <- function(N, p) {
  if (length(N) <= 0) stop("length(N) <= 0")
  if (length(N) != length(p)) stop("length(N) != length(p)")
  if (any(N <= 0)) stop("any(N <= 0)")
  if (any(p < 0)) stop("any(p < 0)")

  N.sum <- sum(N)
  rel <- N / N.sum
  d <- -2 * sum( N * log(p/rel) )

  cat("Deviance comparing relative frequencies with predicted probabilities:\n")
  cat("N = (", paste(N, collapse=", "), ")\n", sep="")
  cat("p = (", paste(round(p, 5), collapse=", "), ")\n", sep="")
  cat("d = ", round(d, 3), "\n\n", sep="")

  return(d)
}

log.multinomial.probability.with.predicted <- function(N, p) {
  if (length(N) <= 0) stop("length(N) <= 0")
  if (length(N) != length(p)) stop("length(N) != length(p)")
  if (any(N <= 0)) stop("any(N <= 0)")
  if (any(p <= 0)) stop("any(p <= 0)")

  N.sum <- sum(N)

  # log(factorial(N.sum)) = lfactorial(N.sum)  
  # log(prod(factorial(N))) = sum(log(factorial(N))) = sum(lfactorial(N))
  log.p <- lfactorial(N.sum) - sum(lfactorial(N)) + sum(N * log(p))
  p <- exp(log.p)

  cat("Log multinomial probability:\n")
  cat("log(p) = ", round(log.p, 6), "\n", sep="")
  cat("p      = ", round(p, 6), "\n\n", sep="")
  
  return(list(p = p, log.p = log.p))
}

