transform.data <- function(dataset.name) {
  loci <- NULL
  dataset <- NULL   

  # dane or somali
  if (dataset.name %in% c("dane", "somali")) {
    dataset <- read.csv2(paste("data/data-original-", dataset.name, ".txt", sep=""), sep=" ")    
    loci <- 1:10

    # Remove No and DYS385a/b
    dataset <- dataset[,-1]
    dataset <- dataset[,-2]
  }

  # Berlin
  else if (dataset.name == "berlin") {
    dataset <- read.csv2(paste("data/data-original-berlin.csv", sep=""), sep=",")
    loci <- 1:7

    # Each row is an observation
    dataset$N <- 1
  }

  else {
    stop("Unknown dataset")
  }

  dataset.length <- nrow(dataset)
  N.column <- length(loci) + 1

  unique.dataset <- unique(apply(dataset[, loci], 1, paste, collapse=","))
  M <- length(unique.dataset)
  dataset.new <- matrix(0, nrow=M, ncol=N.column)

  for (i in 1:M) {
    h.unique <- unique.dataset[i]
    dataset.new[i, loci] <- as.numeric(strsplit(h.unique, ",")[[1]])
    
    for (j in 1:dataset.length) {
      h2 <- paste(dataset[j, loci], collapse=",")

      if (all(h.unique == h2)) {
        dataset.new[i, N.column] <- as.numeric(dataset.new[i, N.column]) + dataset[j, N.column]
      }
    }
  }

  colnames(dataset.new) <- colnames(dataset[, 1:N.column])
  dataset <- as.data.frame(dataset.new)

  N <- sum(dataset$N)
  
  dataset <- cbind(dataset, Freq = dataset$N / N)
  write.csv2(dataset, paste("data/data-compact-", dataset.name, ".csv", sep=""))

  # Unecessary for berlin, but let us just keep because it because of consistancy
  dataset.expanded <- dataset

  for (i in 1:nrow(dataset)) {
    while(dataset.expanded[i, N.column] > 1) {
      dataset.expanded <- rbind(dataset.expanded, dataset.expanded[i,])
      dataset.expanded[i, N.column] <- dataset.expanded[i, N.column] - 1
    }
  }

  dataset.expanded <- dataset.expanded[, loci]
  rownames(dataset.expanded) <- 1:N

  write.csv2(dataset.expanded, paste("data/data-extended-", dataset.name, ".csv", sep=""))

  return(TRUE)
}

