logfile <- "log/model-based-clustering.log"
sink(logfile)

library(mclust)
library(mvtnorm)
library(xtable)

#models <- c("EEE")
#models <- c("EEE", "VVV")
#models <- c("EII", "VII", "EEI", "EVI", "VEI", "VVI")
models <- c("EII", "VII", "EEI", "EVI", "VEI", "VVI", "EEE", "EEV", "VEV", "VVV")
models.length <- length(models)

source("start.R")
source("include-model-based-clustering.R")
source("include-marginal-logic.R")

model.properties.clusters.matrix <- matrix(NA, 
  nrow=length(dataset.names), 
  ncol=length(models), 
  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), models))

model.properties.bic.matrix <- matrix(NA, 
  nrow=length(dataset.names), 
  ncol=length(models), 
  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), models))
  
unobserved.prob.matrix <- matrix(NA, 
  nrow=length(dataset.names), 
  ncol=models.length + 2, 
  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), c("Estimate", "CI", models)))

deviance.relative.predicted.matrix <- matrix(NA, 
  nrow=length(dataset.names), 
  ncol=models.length, 
  dimnames=list(paste("¤DATASET:", dataset.names, "¤", sep=""), models))

# FIXME: Could be put into a general file since other methods does the same as this
unobserved.prob.estimates <- c("0.364", "0.602", "0.277")
unobserved.prob.intervals <- c("[0.321; 0.408]", "[0.510; 0.694]", "[0.212; 0.342]")
unobserved.prob.matrix[, 1] <- unobserved.prob.estimates
unobserved.prob.matrix[, 2] <- unobserved.prob.intervals

for (i in 1:length(dataset.names)) {
#for (i in 2) {
  dataset.name <- dataset.names[i]

  cat("Starting classification for dataset \"", dataset.name, "\".\n", sep="")

  dataset.extended <- get.extended.dataset(dataset.name)
  dataset.compact <- get.compact.dataset(dataset.name)
  dataset.compact.trimmed <- dataset.compact[, 1:get.loci.count(dataset.name)]  

  for (k in 1:models.length) {
    sink()
    cat(".")
    sink(logfile, append = TRUE)
    
    model <- models[k]

    cat("Using model \"", model, "\".\n", sep="")
    
    # Global variable
    # Use 
    # fit <- Mclust(dataset.extended, G=1:20, modelNames=c("EEE", "VVV"))
    # instead?

    fit <- NULL

    resultTry <- try(
      fit <- Mclust(dataset.extended, G=1:20, modelNames=model)
    )

    if (is.null(fit) || inherits(resultTry, "try-error")) {
      cat("Model \"", model, "\" failed.\n\n", sep="")
      next
    }

    if (fit$G == 1) {
      cat("Model \"", model, "\" could only fit with one cluster, skipping.\n\n", sep="")
      next
    }
    
    probs <- apply(dataset.compact.trimmed, 1, g)
    probs.sum <- sum(probs)    

    model.properties.clusters.matrix[i, k] <- fit$parameters$variance$G
    model.properties.bic.matrix[i, k] <- fit$bic
    unobserved.prob.matrix[i, k+2] <- round(1 - probs.sum, 3)
    deviance.relative.predicted.matrix[i, k] <- deviance.relfreq.predicted(dataset.compact$N, probs)

    sink()
    cat("Model = ", model, "\n")
    #cat(paste(round(probs, 8), collapse=", "), "\n")
    cat("sum   = ", probs.sum, "\n")
    cat("unob  = ", unobserved.prob.matrix[i, k+2], "\n")
    sink(logfile, append = TRUE)
    

    fit.pca <- princomp(dataset.extended, cor=TRUE)
    x <- fit.pca$scores[, 1]
    y <- fit.pca$scores[, 2]
    norm.prob <- probs / probs.sum

    cluster <- fit$classification
    cols <- sample(rainbow(fit$parameters$variance$G))

    xlab.txt <- "SPC1"
    ylab.txt <- "SPC2"
    main.txt <- paste("Model based clustering of ", dataset.name, " using ", model, " model", sep="")
    
    pdf(paste("figures/model-based-clustering/", dataset.name, "-prob-", model, "-combined.pdf", sep=""))
    plot(x, y, xlab=xlab.txt, ylab=xlab.txt, main=main.txt, cex=norm.prob*100, pch=3, col=cols[cluster])
    points(x, y, pch=as.character(cluster), cex=0.7)
    dev.off()

    pdf(paste("figures/model-based-clustering/", dataset.name, "-prob-", model, "-prob.pdf", sep=""))
    plot(x, y, xlab=xlab.txt, ylab=xlab.txt, main=main.txt, cex=norm.prob*100, pch=3, col=cols[cluster])
    dev.off()

    pdf(paste("figures/model-based-clustering/", dataset.name, "-prob-", model, "-cluster.pdf", sep=""))
    plot(x, y, xlab=xlab.txt, ylab=xlab.txt, main=main.txt, pch=as.character(cluster))
    dev.off()

    pdf(paste("figures/model-based-clustering/", dataset.name, "-rel-vs-smoothed-freq-normalised-", model, ".pdf", sep=""))
    # Normalise to get comparable lengths of the lines
    rel.freq <- dataset.compact$Freq * probs.sum
    scale <- 15
    plot(x, y, xlab=xlab.txt, ylab=xlab.txt, main=main.txt, cex=0, pch=21)
    for (m in 1:nrow(dataset.compact.trimmed)) {
      lines(c(x[m] - scale * rel.freq[m], x[m] + scale * rel.freq[m]), rep(y[m], 2), col="red")
      lines(rep(x[m], 2), c(y[m] - scale * probs[m], y[m] + scale * probs[m]), col="blue")
    }    
    legend("topright", c("Relative frequency", "Normalised predicted prob."), col=c("red", "blue"), pch=c(21, 21))  
    dev.off()
    
    pdf(paste("figures/model-based-clustering/", dataset.name, "-rel-vs-smoothed-freq-", model, ".pdf", sep=""))
    # Normalise to get comparable lengths of the lines
    rel.freq <- dataset.compact$Freq
    scale <- 15
    plot(x, y, xlab=xlab.txt, ylab=xlab.txt, main=main.txt, cex=0, pch=21)
    for (m in 1:nrow(dataset.compact.trimmed)) {
      lines(c(x[m] - scale * rel.freq[m], x[m] + scale * rel.freq[m]), rep(y[m], 2), col="red")
      lines(rep(x[m], 2), c(y[m] - scale * probs[m], y[m] + scale * probs[m]), col="blue")
    }    
    legend("topright", c("Relative frequency", "Normalised predicted prob."), col=c("red", "blue"), pch=c(21, 21))  
    dev.off()

    pdf(paste("figures/model-based-clustering/", dataset.name, "-rel-vs-pred-", model, ".pdf", sep=""))
    y.min <- min(c(dataset.compact$Freq, norm.prob))
    y.max <- max(c(dataset.compact$Freq, norm.prob))
    pchs <- c(21, 24)
    plot(dataset.compact$Freq, col="red", ylim = c(y.min, y.max), ylab="Frequency", pch=pchs[1])
    points(norm.prob, col="blue", pch=pchs[2])
    grid(50, 20, col="black")
    legend("topright", c("Relative frequency", "Normalised predicted prob."), col=c("red", "blue"), pch=pchs)
    dev.off()

    #for (coord.type in c("Classification", "Uncertainty", "Errors")) {
    for (coord.type in c("Classification", "Uncertainty")) {
      pdf(paste("figures/model-based-clustering/", dataset.name, "-coord-", tolower(coord.type), "-", model, ".pdf", sep=""))
      coordProj(data = cbind(SPC1=x, SPC2=y), what = tolower(coord.type), parameters = fit$parameters, z = fit$z)
      title(main=main.txt, sub=coord.type)
      dev.off()
    }

    #surfacePlot(data = cbind(SPC1=x, SPC2=y), what = "density", type = "contour", parameters = fit$parameters, transformation = "sqrt")

    cat("Model \"", model, "\" done.\n\n", sep="")
  }  

  sink()
  cat("\n")
  sink(logfile, append = TRUE)

  cat("Dataset \"", dataset.name, "\" done.\n\n", sep="")  
}

sink()

unobserved.prob.matrix <- t(unobserved.prob.matrix)
tableOutput <- xtable(unobserved.prob.matrix,
      align=c("l", rep("r", ncol(unobserved.prob.matrix))),
      label=paste("tab:model-based-clustering-unobserved-probabilities", sep=""), 
      caption=paste("Unobserved probabilities using model based clustering. The missing values indicate that the method did not succeed for some reason."))
filename <- paste("out/model-based-clustering/unobserved-probabilities.tex", sep="")
print(tableOutput, file=filename)
do.replacements(filename)

deviance.relative.predicted.matrix <- t(deviance.relative.predicted.matrix)
tableOutput <- xtable(deviance.relative.predicted.matrix,
      align=c("l", rep("r", ncol(deviance.relative.predicted.matrix))),
      label=paste("tab:model-based-clustering-deviance-relative-predicted", sep=""), 
      caption=paste("Comparing deviance between relative frequencies and predicted probabilities found using model-based-clustering. The missing values indicate that the method did not succeed for some reason."))
filename <- paste("out/model-based-clustering/deviance-relative-predicted.tex", sep="")
print(tableOutput, file=filename)
do.replacements(filename)

model.properties.clusters.matrix <- t(model.properties.clusters.matrix)
tableOutput <- xtable(model.properties.clusters.matrix,
      align=c("l", rep("r", ncol(model.properties.clusters.matrix))),
      label=paste("tab:model-based-clustering-properties-clusters", sep=""), 
      caption=paste("Number of clusters used. The missing values indicate that the method did not succeed for some reason."))
filename <- paste("out/model-based-clustering/model-based-clustering-properties-clusters.tex", sep="")
print(tableOutput, file=filename)
do.replacements(filename)

model.properties.bic.matrix <- t(model.properties.bic.matrix)
tableOutput <- xtable(model.properties.bic.matrix,
      align=c("l", rep("r", ncol(model.properties.bic.matrix))),
      label=paste("tab:model-based-clustering-properties-bic", sep=""), 
      caption=paste("The BIC-score for each model. The missing values indicate that the method did not succeed for some reason."))
filename <- paste("out/model-based-clustering/model-based-clustering-properties-bic.tex", sep="")
print(tableOutput, file=filename)
do.replacements(filename)

